# -*- coding: utf-8 -*-
"""Logging utilities for acme2certifier"""

import logging
import sys
import copy
from typing import Dict
import datetime
from .config import load_config


def _logger_nonce_modify(data_dic: Dict[str, str]) -> Dict[str, str]:
    """remove nonce from log entry"""
    if "header" in data_dic and "Replay-Nonce" in data_dic["header"]:
        data_dic["header"]["Replay-Nonce"] = "- modified -"
    return data_dic


def _logger_certificate_modify(
    data_dic: Dict[str, str], locator: str
) -> Dict[str, str]:
    """remove cert from log entry"""
    if "/acme/cert" in locator:
        data_dic["data"] = " - certificate - "
    return data_dic


def _logger_token_modify(data_dic: Dict[str, str]) -> Dict[str, str]:
    """remove token from challenge"""
    if "token" in data_dic["data"]:
        data_dic["data"]["token"] = "- modified -"
    return data_dic


def _logger_challenges_modify(data_dic: Dict[str, str]) -> Dict[str, str]:
    """remove token from challenge"""
    if "challenges" in data_dic["data"]:
        for challenge in data_dic["data"]["challenges"]:
            if "token" in challenge:
                challenge.update(
                    (k, "- modified - ") for k, v in challenge.items() if k == "token"
                )
    return data_dic


def logger_info(
    logger: logging.Logger, addr: str, locator: str, dat_dic: Dict[str, str]
):
    """log responses"""
    # create a copy of the dictionary
    data_dic = copy.deepcopy(dat_dic)

    data_dic = _logger_nonce_modify(data_dic)
    if "data" in data_dic:
        # remove cert from log entry
        data_dic = _logger_certificate_modify(data_dic, locator)

        # remove token
        data_dic = _logger_token_modify(data_dic)

        # remove token from challenge
        data_dic = _logger_challenges_modify(data_dic)

    logger.info("%s %s %s", addr, locator, str(data_dic))


def logger_setup(debug: bool) -> logging.Logger:
    """setup logger"""
    if debug:
        log_mode = logging.DEBUG
    else:
        log_mode = logging.INFO

    config_dic = load_config()

    # define standard log format
    log_format = "%(message)s"
    if "Helper" in config_dic and "log_format" in config_dic["Helper"]:
        log_format = config_dic["Helper"]["log_format"]

    logging.basicConfig(format=log_format, datefmt="%Y-%m-%d %H:%M:%S", level=log_mode)
    logger = logging.getLogger("acme2certifier")
    return logger


def print_debug(debug: bool, text: str):
    """little helper to print debug messages"""
    if debug:
        print(f"{datetime.datetime.now()}: {text}")


def handle_exception(exc_type, exc_value, exc_traceback):  # pragma: no cover
    """exception handler"""
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return

    logging.exception(
        "Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback)
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""django handler for acme2certifier"""

# pylint: disable=c0413, c0415, c0401, e0401, e1123, r0904, w0611
from __future__ import print_function
import os
import sys
import json
from datetime import datetime, timezone
from typing import List, Tuple, Dict


def initialize():  # nopep8
    """initialize routine when calling dbstore functions from script"""
    sys.path.append(
        os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir))
    )
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE", "acme2certifier.django_project.settings"
    )
    import django

    # pylint: disable=E1101
    django.setup()
    return django.VERSION[0]


DJANGO_VERSION = initialize()
from django.conf import settings  # nopep8
from django.db import transaction  # nopep8
from django.db.models import QuerySet  # nopep8
from acme2certifier.django_app.models import (
    Account,
    Authorization,
    Cahandler,
    Certificate,
    Challenge,
    Cliaccount,
    Housekeeping,
    Nonce,
    Order,
    Status,
)  # nopep8

if DJANGO_VERSION < 4:
    import acme2certifier.acme_srv.monkey_patches  # nopep8 lgtm [py/unused-import]


class DBstore(object):
    """helper to do datebase operations"""

    def __init__(self, _debug: bool = False, logger: object = None):
        """init"""
        self.logger = logger
        self.type = "django"

    def _account_getinstance(self, aname: str) -> QuerySet:
        """get account instance"""
        self.logger.debug("DBStore._account_getinstance(%s)", aname)
        return Account.objects.get(name=aname)

    def _authorization_getinstance(self, name: str) -> QuerySet:
        """get authorization instance"""
        self.logger.debug("DBStore._authorization_getinstance(%s)", name)
        return Authorization.objects.get(name=name)

    def _modify_key(self, mkey: str, operant: str) -> str:
        """quick hack"""
        self.logger.debug("DBStore._modify_key(%s/%s)", mkey, operant)

        if operant == "<=":
            mkey = f"{mkey}__lte"

        self.logger.debug("DBStore._modify_key() ended with: %s", mkey)
        return mkey

    def _order_getinstance(self, value: str = id, mkey: id = "id") -> QuerySet:
        """get order instance"""
        self.logger.debug("DBStore._order_getinstance(%s:%s)", mkey, value)
        return Order.objects.get(**{mkey: value})

    def _status_getinstance(self, value: str, mkey: str = "id") -> QuerySet:
        """get account instance"""
        self.logger.debug("DBStore._status_getinstance(%s:%s)", mkey, value)
        return Status.objects.get(**{mkey: value})

    def account_add(self, data_dic: Dict[str, str]) -> Tuple[str, bool]:
        """add account in database"""
        self.logger.debug("DBStore.account_add(%s)", data_dic)
        account_list = self.account_lookup("jwk", data_dic["jwk"])
        if "status" in data_dic:
            data_dic["status"] = self._status_getinstance(data_dic["status"], "name")

        if "eab_kid" in data_dic and not data_dic["eab_kid"]:
            del data_dic["eab_kid"]

        if account_list:
            created = False
            aname = account_list["name"]
        else:
            obj, created = Account.objects.update_or_create(
                name=data_dic["name"], defaults=data_dic
            )
            obj.save()
            aname = data_dic["name"]
        return aname, created

    def account_lookup(
        self,
        mkey: str,
        value: str,
        vlist: List = [
            "id",
            "jwk",
            "name",
            "contact",
            "alg",
            "created_at",
            "eab_kid",
            "status_id",
        ],
    ) -> Dict[str, str]:
        """search account for a given id"""
        self.logger.debug("DBStore.account_lookup(%s:%s)", mkey, value)
        account_dict = Account.objects.filter(**{mkey: value}).values(*vlist)[:1]
        if account_dict.exists():
            result = account_dict[0]
        else:
            result = None
        return result

    def account_delete(self, aname):
        """add account in database"""
        self.logger.debug("DBStore.account_delete(%s)", aname)
        result = Account.objects.filter(name=aname).delete()
        return result

    def account_update(
        self,
        data_dic: Dict[str, str],
        active: bool = True,  # NOSONAR # pylint: disable=unused-argument
    ) -> int:
        """update existing account"""
        self.logger.debug("DBStore.account_update(%s)", data_dic)
        obj, _created = Account.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()
        self.logger.debug("acct_id(%s)", obj.id)
        return obj.id

    def accountlist_get(self) -> Tuple[List[str], QuerySet]:
        """accountlist_get"""
        self.logger.debug("DBStore.accountlist_get()")
        vlist = [
            "id",
            "name",
            "contact",
            "eab_kid",
            "created_at",
            "jwk",
            "alg",
            "order__id",
            "order__name",
            "order__status__id",
            "order__status__name",
            "order__notbefore",
            "order__notafter",
            "order__expires",
            "order__identifiers",
            "order__authorization__id",
            "order__authorization__name",
            "order__authorization__type",
            "order__authorization__value",
            "order__authorization__expires",
            "order__authorization__token",
            "order__authorization__created_at",
            "order__authorization__status_id",
            "order__authorization__status__id",
            "order__authorization__status__name",
            "order__authorization__challenge__id",
            "order__authorization__challenge__name",
            "order__authorization__challenge__token",
            "order__authorization__challenge__expires",
            "order__authorization__challenge__type",
            "order__authorization__challenge__keyauthorization",
            "order__authorization__challenge__created_at",
            "order__authorization__challenge__status__id",
            "order__authorization__challenge__status__name",
        ]
        # for historical reason cert_raw an be NULL or ''; we have to consider both cases during selection
        return vlist, list(Account.objects.filter(name__isnull=False).values(*vlist))

    def authorization_add(self, data_dic: Dict[str, str]) -> int:
        """add authorization to database"""
        self.logger.debug("DBStore.authorization_add(%s)", data_dic)

        # get some instance for DB insert
        if "order" in data_dic:
            data_dic["order"] = self._order_getinstance(data_dic["order"], "id")
        if "status" in data_dic:
            data_dic["status"] = self._status_getinstance(data_dic["status"], "name")

        # add authorization
        obj, _created = Authorization.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()
        self.logger.debug("auth_id(%s)", obj.id)
        return obj.id

    def authorization_lookup(
        self, mkey: str, value: str, vlist: List[str] = ("type", "value")
    ) -> QuerySet:
        """search account for a given id"""
        self.logger.debug("authorization_lookup(%s:%s:%s)", mkey, value, vlist)
        authz_list = Authorization.objects.filter(**{mkey: value}).values(*vlist)[::1]
        return authz_list

    def authorizations_expired_search(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = (
            "id",
            "name",
            "expires",
            "identifiers",
            "created_at",
            "status__id",
            "status__name",
            "account__id",
            "account__name",
            "acccount__contact",
        ),
        operant: str = "LIKE",
    ) -> QuerySet:
        """search order table for a certain key/value pair"""
        self.logger.debug(
            "DBStore.authorizations_invalid_search(column:%s, pattern:%s)", mkey, value
        )

        mkey = self._modify_key(mkey, operant)

        self.logger.debug("DBStore.authorizations_invalid_search() ended")
        return (
            Authorization.objects.filter(**{mkey: value})
            .exclude(status__name="expired")
            .values(*vlist)
        )

    def authorization_update(self, data_dic: Dict[str, str]) -> int:
        """update existing authorization"""
        self.logger.debug("DBStore.authorization_update(%s)", data_dic)
        # get some instance for DB insert
        if "status" in data_dic:
            data_dic["status"] = self._status_getinstance(data_dic["status"], "name")

        if (
            DJANGO_VERSION < 4
            and settings.DATABASES["default"]["ENGINE"] == "django.db.backends.sqlite3"
        ):
            self.logger.debug(
                "DBStore.authorization_update(): patching transaction to transform all atomic blocks into immediate transactions"
            )
            with transaction.atomic(immediate=True):
                # update authorization
                obj, _created = Authorization.objects.update_or_create(
                    name=data_dic["name"], defaults=data_dic
                )
                obj.save()
        else:
            # update authorization
            obj, _created = Authorization.objects.update_or_create(
                name=data_dic["name"], defaults=data_dic
            )
            obj.save()

        self.logger.debug("auth_id(%s)", obj.id)
        return obj.id

    def cahandler_add(self, data_dic: Dict[str, str]) -> Tuple[str, bool]:
        """add cahandler to database"""
        self.logger.debug("DBStore.cahandler_add(%s)", data_dic)
        cahandler_list = self.cahandler_lookup("name", data_dic["name"])
        if cahandler_list:
            created = False
            cname = cahandler_list["name"]
        else:
            obj, created = Cahandler.objects.update_or_create(
                name=data_dic["name"], defaults=data_dic
            )
            obj.save()
            cname = data_dic["name"]
        return (cname, created)

    def cahandler_lookup(self, mkey: str, value: str) -> Dict[str, str]:
        """search cahandler for a given id"""
        self.logger.debug("DBStore.cahandler_lookup(%s:%s)", mkey, value)
        cahandler_dict = Cahandler.objects.filter(**{mkey: value}).values(
            "name", "value1", "value2", "created_at"
        )[:1]
        if cahandler_dict.exists():
            result = cahandler_dict[0]
        else:
            result = None
        return result

    def challenge_add(self, value: str, mtype: str, data_dic: Dict[str, str]) -> int:
        """add challenge to database"""
        self.logger.debug("DBStore.challenge_add(%s:%s)", value, mtype)

        # get order instance for DB insert
        data_dic["authorization"] = self._authorization_getinstance(
            data_dic["authorization"]
        )

        # replace orderstatus with an instance
        data_dic["status"] = self._status_getinstance(data_dic["status"])

        if (
            DJANGO_VERSION < 4
            and settings.DATABASES["default"]["ENGINE"] == "django.db.backends.sqlite3"
        ):
            self.logger.debug(
                "DBStore.challenge_add(): patching transaction to transform all atomic blocks into immediate transactions"
            )
            with transaction.atomic(immediate=True):
                obj, _created = Challenge.objects.update_or_create(
                    name=data_dic["name"], defaults=data_dic
                )
                obj.save()
        else:
            obj, _created = Challenge.objects.update_or_create(
                name=data_dic["name"], defaults=data_dic
            )
            obj.save()

        self.logger.debug("cid(%s)", obj.id)
        self.logger.debug("DBStore.challenge_add(%s:%s:%s)", value, mtype, obj.id)
        return obj.id

    def certificate_add(self, data_dic: Dict[str, str]) -> int:
        """add csr/certificate to database"""
        self.logger.debug("DBStore.certificate_add()")

        # get order instance for DB insert
        if "order" in data_dic:
            data_dic["order"] = self._order_getinstance(data_dic["order"], "name")
        # add certificate/CSR
        obj, _created = Certificate.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()
        self.logger.debug("DBStore.certificate_add() ended with :%s", obj.id)
        return obj.id

    def certificate_account_check(self, account_name: str, certificate: str) -> str:
        """check issuer against certificate"""
        self.logger.debug("DBStore.certificate_account_check(%s)", account_name)

        result = None
        certificate_list = self.certificate_lookup(
            "cert_raw", certificate, ["name", "order__name", "order__account__name"]
        )

        if certificate_list:
            if account_name:
                # if there is an acoount name validate it against the account_name from db-query
                if account_name == certificate_list["order__account__name"]:
                    result = certificate_list["order"]
            else:
                # no account name given (message signed with domain key
                result = certificate_list["order"]

        self.logger.debug("DBStore.certificate_account_check() ended with: %s", result)
        return result

    def certificate_delete(self, mkey: str, value: str) -> QuerySet:
        """delete certificate from table"""
        self.logger.debug("DBStore.certificate_delete(%s:%s)", mkey, value)
        Certificate.objects.filter(**{mkey: value}).delete()

    def certificatelist_get(self) -> Tuple[List[str], List[QuerySet]]:
        """certificatelist_get"""
        self.logger.debug("DBStore.certificatelist_get()")
        vlist = [
            "id",
            "name",
            "cert_raw",
            "csr",
            "poll_identifier",
            "created_at",
            "issue_uts",
            "expire_uts",
            "order__id",
            "order__name",
            "order__status__name",
            "order__notbefore",
            "order__notafter",
            "order__expires",
            "order__identifiers",
            "order__account__name",
            "order__account__contact",
            "order__account__created_at",
            "order__account__jwk",
            "order__account__alg",
            "order__account__eab_kid",
        ]
        # for historical reason cert_raw an be NULL or ''; we have to consider both cases during selection
        return (
            vlist,
            list(
                Certificate.objects.filter(cert_raw__isnull=False)
                .exclude(cert_raw="")
                .values(*vlist)
            ),
        )

    def certificate_lookup(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = ("name", "csr", "cert", "order__name"),
    ) -> Dict[str, str]:
        """search certificate based on "something" """
        self.logger.debug("DBStore.certificate_lookup(%s:%s)", mkey, value)
        certificate_list = Certificate.objects.filter(**{mkey: value}).values(*vlist)[
            :1
        ]
        if certificate_list:
            result = certificate_list[0]
            if "order__name" in result:
                result["order"] = result["order__name"]
                del result["order__name"]
        else:
            result = None
        self.logger.debug("DBStore.certificate_lookup() ended with: %s", result)
        return result

    def certificates_search(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = ("name", "csr", "cert", "order__name"),
        operant=None,
    ) -> QuerySet:
        """search certificate based on "something" """
        self.logger.debug("DBStore.certificates_search(%s:%s)", mkey, value)
        mkey = self._modify_key(mkey, operant)
        return Certificate.objects.filter(**{mkey: value}).values(*vlist)

    def challenge_lookup(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = ("type", "token", "status__name"),
    ) -> Dict[str, str]:
        """search account for a given id"""
        self.logger.debug("DBStore.challenge_lookup(%s:%s)", mkey, value)
        challenge_list = Challenge.objects.filter(**{mkey: value}).values(*vlist)[:1]
        if challenge_list:
            result = challenge_list[0]
            if "status__name" in result:
                result["status"] = result["status__name"]
                del result["status__name"]
            if "authorization__name" in result:
                result["authorization"] = result["authorization__name"]
                del result["authorization__name"]
        else:
            result = None
        return result

    def challenges_search(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = ("name", "type", "cert", "status__name", "token"),
    ) -> QuerySet:
        """search challenges based on "something" """
        self.logger.debug("DBStore.challenges_search(%s:%s)", mkey, value)
        return Challenge.objects.filter(**{mkey: value}).values(*vlist)

    def challenge_update(self, data_dic: Dict[str, str]):
        """update challenge"""
        self.logger.debug("challenge_update(%s)", data_dic)
        # replace orderstatus with an instance
        if "status" in data_dic:
            data_dic["status"] = self._status_getinstance(data_dic["status"], "name")
        obj, _created = Challenge.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()

    def cli_jwk_load(self, aname: str) -> Dict[str, str]:
        """looad account informatino and build jwk key dictionary from cliaccounts teable"""
        self.logger.debug("DBStore.cli_jwk_load(%s)", aname)
        account_dict = Cliaccount.objects.filter(name=aname).values("jwk")[:1]
        jwk_dict = {}
        if account_dict:
            try:
                jwk_dict = json.loads(account_dict[0]["jwk"].decode())
            except Exception as _err:
                self.logger.error(
                    "Failed to decode JWK from cliaccounts table: %s", _err
                )
                jwk_dict = json.loads(account_dict[0]["jwk"])
        return jwk_dict

    def cli_permissions_get(self, aname: str) -> Dict[str, str]:
        """looad account informations and build jwk key dictionary from cliaccounts teable"""
        self.logger.debug("DBStore.cli_jwk_load(%s)", aname)
        account_dict = Cliaccount.objects.filter(name=aname).values(
            "reportadmin", "cliadmin", "certificateadmin"
        )[:1]
        permissions_dict = {}
        if account_dict.exists():
            permissions_dict = account_dict[0]
        return permissions_dict

    def dbversion_get(self) -> Tuple[Dict[str, str], str]:
        """get db version from housekeeping table"""
        self.logger.debug("DBStore.dbversion_get()")
        version_list = Housekeeping.objects.filter(name="dbversion").values_list(
            "value", flat=True
        )
        if version_list:
            result = version_list[0]
        else:
            result = None
        self.logger.debug("DBStore.dbversion_get() ended with %s", result)
        return (result, "a2c-django-update")

    def hkparameter_add(self, data_dic: Dict[str, str]):
        """add housekeeping paramter to database"""
        self.logger.debug("DBStore.hkparameter_add(%s)", data_dic)
        obj, _created = Housekeeping.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()

    def hkparameter_get(self, parameter: str) -> str:
        """get parameter from housekeeping table"""
        self.logger.debug("DBStore.hkparameter_get()")
        result_list = Housekeeping.objects.filter(name=parameter).values_list(
            "value", flat=True
        )
        if result_list:
            result = result_list[0]
        else:
            result = None
        self.logger.debug("DBStore.hkparameter_get() ended with %s", result)
        return result

    def jwk_load(self, aname: str) -> Dict[str, str]:
        """looad account informatino and build jwk key dictionary"""
        self.logger.debug("DBStore.jwk_load(%s)", aname)
        account_dict = Account.objects.filter(name=aname, status_id=5).values(
            "jwk", "alg"
        )[:1]
        jwk_dict = {}
        if account_dict:
            try:
                jwk_dict = json.loads(account_dict[0]["jwk"].decode())
            except Exception:
                jwk_dict = json.loads(account_dict[0]["jwk"])
            jwk_dict["alg"] = account_dict[0]["alg"]
        return jwk_dict

    def nonce_add(self, nonce: str) -> int:
        """check if nonce is in datbase
        in: nonce
        return: rowid"""
        self.logger.debug("DBStore.nonce_add(%s)", nonce)
        obj = Nonce(nonce=nonce)
        obj.save()
        return obj.id

    def nonce_check(self, nonce: str) -> bool:
        """ceck if nonce is in datbase
        in: nonce
        return: true in case nonce exit, otherwise false"""
        self.logger.debug("DBStore.nonce_check(%s)", nonce)
        nonce_list = Nonce.objects.filter(nonce=nonce).values("nonce")[:1]
        return bool(nonce_list)

    def nonce_delete(self, nonce: str):
        """delete nonce from datbase
        in: nonce"""
        self.logger.debug("DBStore.nonce_delete(%s)", nonce)
        Nonce.objects.filter(nonce=nonce).delete()

    def nonce_delete_bulk(self, nonce_list: List[str]) -> int:
        """Delete a list of nonces in a single cleanup run."""
        self.logger.debug("DBStore.nonce_delete_bulk with %s entries", len(nonce_list))
        if not nonce_list:
            return 0
        result, _ = Nonce.objects.filter(nonce__in=nonce_list).delete()
        return result

    def nonce_search_by_timestamp(self, timestamp: int) -> List[str]:
        """search nonce table for a certain timestamp"""
        dt = datetime.fromtimestamp(timestamp, tz=timezone.utc)
        self.logger.debug("DBStore.nonce_search_by_timestamp(%s)", timestamp)
        nonce_list = Nonce.objects.filter(created_at__lte=dt).values_list(
            "nonce", flat=True
        )
        return list(nonce_list)

    def order_add(self, data_dic: Dict[str, str]) -> int:
        """add order to database"""
        self.logger.debug("DBStore.order_add(%s)", data_dic)
        # replace accountid with instance
        data_dic["account"] = self._account_getinstance(data_dic["account"])

        # replace orderstatus with an instance
        data_dic["status"] = self._status_getinstance(data_dic["status"], "id")
        obj, _created = Order.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()
        self.logger.debug("order_id(%s)", obj.id)
        return obj.id

    def order_lookup(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = (
            "name",
            "notbefore",
            "notafter",
            "identifiers",
            "status__name",
            "account__name",
            "expires",
        ),
    ) -> Dict[str, str]:
        """search orders for a given ordername"""
        self.logger.debug("order_lookup(%s:%s)", mkey, value)
        order_list = Order.objects.filter(**{mkey: value}).values(*vlist)[:1]
        if order_list:
            result = order_list[0]
            if "status__name" in result:
                result["status"] = result["status__name"]
                del result["status__name"]
            if "account_name" in result:
                result["account"] = result["account__name"]
                del result["account__name"]
        else:
            result = None
        return result

    def order_update(self, data_dic: Dict[str, str]):
        """update order"""
        self.logger.debug("order_update(%s)", data_dic)
        # replace orderstatus with an instance
        if "status" in data_dic:
            data_dic["status"] = self._status_getinstance(data_dic["status"], "name")
        obj, _created = Order.objects.update_or_create(
            name=data_dic["name"], defaults=data_dic
        )
        obj.save()

    def orders_invalid_search(
        self,
        mkey: str,
        value: str,
        vlist: List[str] = (
            "id",
            "name",
            "expires",
            "identifiers",
            "created_at",
            "status__id",
            "status__name",
            "account__id",
            "account__name",
            "acccount__contact",
        ),
        operant="LIKE",
    ) -> QuerySet:
        """search order table for a certain key/value pair"""
        self.logger.debug("DBStore.orders_search(column:%s, pattern:%s)", mkey, value)
        mkey = self._modify_key(mkey, operant)
        return Order.objects.filter(**{mkey: value}, status__id__gt=1).values(*vlist)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""unittests for acme2certifier"""

# pylint: disable= C0415, W0212
import unittest
import sys
import os
from unittest.mock import patch, Mock, MagicMock
import requests
import configparser

sys.path.insert(0, ".")
sys.path.insert(1, "..")


class TestACMEHandler(unittest.TestCase):
    """test class for certifier_ca_handler"""

    def setUp(self):
        """setup unittest"""
        import logging

        logging.basicConfig(level=logging.CRITICAL)
        self.logger = logging.getLogger("test_a2c")
        from acme2certifier.cahandlers.certifier_ca_handler import CAhandler

        self.cahandler = CAhandler(False, self.logger)
        # self.cahandler.api_host = 'api_host'
        # self.cahandler.auth = 'auth'

    def test_001_default(self):
        """default test which always passes"""
        self.assertEqual("foo", "foo")

    @patch("requests.get")
    def test_002_ca_get(self, mock_get):
        """CAhandler.get_ca() returns an http error"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_get.side_effect = requests.exceptions.HTTPError
        self.assertEqual(
            {"status": 500, "message": "", "statusMessage": "Internal Server Error"},
            self.cahandler._ca_get("foo", "bar"),
        )

    @patch("requests.get")
    def test_003_ca_get(self, mock_get):
        """CAhandler.get_ca() returns no json file"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_get.status_code = 200
        mock_get.return_value.json = {"bbs": "hahha"}
        self.assertEqual(
            {
                "status": 500,
                "message": "'dict' object is not callable",
                "statusMessage": "Internal Server Error",
            },
            self.cahandler._ca_get("foo", "bar"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_004_config_load(self, mock_load_cfg):
        """test _config_load no cahandler section"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {}
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_005_config_load(self, mock_load_cfg):
        """test _config_load no api_host parameter"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"foo": "bar"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertIn(
            'ERROR:test_a2c:Configuration incomplete: "api_host" parameter is missing in config file',
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_006_config_load(self, mock_load_cfg):
        """test _config_load no api_user parameter"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"api_host": "api_host", "foo": "bar"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertIn(
            'ERROR:test_a2c:Configuration incomplete: "api_user" parameter is missing in config file',
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_007_config_load(self, mock_load_cfg):
        """test _config_load no api_password parameter"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "foo": "bar",
        }
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertIn(
            'ERROR:test_a2c:Configuration incomplete: "api_password" parameter is missing in config file',
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_008_config_load(self, mock_load_cfg):
        """test _config_load no ca_name parameter"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "foo": "bar",
        }
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertIn(
            'ERROR:test_a2c:Configuration incomplete: "ca_name" parameter is missing in config file',
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_009_config_load(self, mock_load_cfg):
        """test _config_load standard polling interval"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "ca_name": "ca_name",
            "foo": "bar",
        }
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertEqual("ca_name", self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_010_config_load(self, mock_load_cfg):
        """test _config_load custom polling interval"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "ca_name": "ca_name",
            "foo": "bar",
            "polling_timeout": 120,
        }
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertEqual("ca_name", self.cahandler.ca_name)
        self.assertEqual(120, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_011_config_load(self, mock_load_cfg):
        """test _config_load custom polling interval"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "ca_name": "ca_name",
            "foo": "bar",
            "polling_timeout": "aa",
        }
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "WARNING:test_a2c:Invalid value for polling_timeout in configuration. Using default: 60",
            lcm.output,
        )
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertEqual("ca_name", self.cahandler.ca_name)
        self.assertEqual(60, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_012_config_load(self, mock_load_cfg):
        """test _config_load ca_handler True"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "ca_name": "ca_name",
            "foo": "bar",
            "polling_timeout": 120,
            "ca_bundle": True,
        }
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertTrue(self.cahandler.ca_bundle)
        self.assertEqual("ca_name", self.cahandler.ca_name)
        self.assertEqual(120, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_013_config_load(self, mock_load_cfg):
        """test _config_load ca_handler False"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "ca_name": "ca_name",
            "foo": "bar",
            "polling_timeout": 120,
            "ca_bundle": False,
        }
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertEqual("ca_name", self.cahandler.ca_name)
        self.assertEqual(120, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_014_config_load(self, mock_load_cfg):
        """test _config_load ca_handler configured"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_host": "api_host",
            "api_user": "api_user",
            "api_password": "api_password",
            "ca_name": "ca_name",
            "foo": "bar",
            "polling_timeout": 120,
            "ca_bundle": "foo",
        }
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertEqual("foo", self.cahandler.ca_bundle)
        self.assertEqual("ca_name", self.cahandler.ca_name)
        self.assertEqual(120, self.cahandler.polling_timeout)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_015_config_load(self, mock_load_cfg):
        """test _config_load - load template with user variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"api_user_variable": "api_user_var"}
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("user_var", self.cahandler.api_user)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_016_config_load(self, mock_load_cfg):
        """test _config_load - load template with user variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"api_user_variable": "does_not_exist"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertFalse(self.cahandler.api_user)
        self.assertIn(
            "ERROR:test_a2c:Could not load user_variable:'does_not_exist'",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_017_config_load(self, mock_load_cfg):
        """test _config_load - load template with user variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_user_variable": "api_user_var",
            "api_user": "api_user",
        }
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        # with self.assertLogs('test_a2c', level='INFO') as lcm:
        self.assertEqual("api_user", self.cahandler.api_user)
        # self.assertIn("foo", lcm.output)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_password_var": "password_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_018_config_load(self, mock_load_cfg):
        """test _config_load - load template with password variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"api_password_variable": "api_password_var"}
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("password_var", self.cahandler.api_password)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_password_var": "password_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_019_config_load(self, mock_load_cfg):
        """test _config_load - load template with password variable which does not exist"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"api_password_variable": "does_not_exist"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertFalse(self.cahandler.api_password)
        self.assertIn(
            "ERROR:test_a2c:Could not load passphrase_variable:'does_not_exist'",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_password_var": "password_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_020_config_load(self, mock_load_cfg):
        """test _config_load - load template override password variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {
            "api_password_variable": "api_password_var",
            "api_password": "api_password",
        }
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertIn(
            "INFO:test_a2c:Overwrite api_password_variable",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.parse_url")
    @patch("json.loads")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_021_config_load(self, mock_load_cfg, mock_json, mock_url):
        """test _config_load ca_handler configured load proxies"""
        parser = configparser.ConfigParser()
        parser["DEFAULT"] = {"proxy_server_list": "foo"}
        mock_load_cfg.return_value = parser
        mock_url.return_value = {"foo": "bar"}
        mock_json.return_value = "foo"
        self.cahandler._config_load()
        self.assertTrue(mock_json.called)
        self.assertTrue(mock_url.called)
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.proxy_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.parse_url")
    @patch("json.loads")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_022_config_load(self, mock_load_cfg, mock_json, mock_url, mock_chk):
        """test _config_load ca_handler configured load proxies"""
        parser = configparser.ConfigParser()
        parser["DEFAULT"] = {"proxy_server_list": "foo"}
        mock_load_cfg.return_value = parser
        mock_url.return_value = {"host": "bar:8888"}
        mock_json.return_value = "foo.bar.local"
        mock_chk.return_value = "proxy.bar.local"
        self.cahandler._config_load()
        self.assertTrue(mock_json.called)
        self.assertTrue(mock_url.called)
        self.assertTrue(mock_chk.called)
        self.assertEqual(
            {"http": "proxy.bar.local", "https": "proxy.bar.local"},
            self.cahandler.proxy,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.proxy_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.parse_url")
    @patch("json.loads")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_023_config_load(self, mock_load_cfg, mock_json, mock_url, mock_chk):
        """test _config_load ca_handler configured load proxies"""
        parser = configparser.ConfigParser()
        parser["DEFAULT"] = {"proxy_server_list": "foo"}
        mock_load_cfg.return_value = parser
        mock_url.return_value = {"host": "bar"}
        mock_json.return_value = "foo.bar.local"
        mock_chk.return_value = "proxy.bar.local"
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertTrue(mock_json.called)
        self.assertTrue(mock_url.called)
        self.assertFalse(mock_chk.called)
        self.assertFalse(self.cahandler.proxy)
        self.assertIn(
            "WARNING:test_a2c:Failed to parse proxy_server_list from configuration: not enough values to unpack (expected 2, got 1)",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_024_config_load(self, mock_load_cfg):
        """test _config_load - load template with timeout variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"request_timeout": 10}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_025_config_load(self, mock_load_cfg):
        """test _config_load - load template with timeout variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"request_timeout": "aa"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "WARNING:test_a2c:Invalid value for request_timeout in configuration. Using default: 20",
            lcm.output,
        )
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_026_config_load(self, mock_load_cfg):
        """test _config_load - load template with timeout variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"request_timeout": "aa"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertFalse(self.cahandler.profile_id)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    @patch("acme2certifier.cahandlers.certifier_ca_handler.load_config")
    def test_027_config_load(self, mock_load_cfg):
        """test _config_load - load template with user variable"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"profile_id": "profile_id"}
        mock_load_cfg.return_value = parser
        self.cahandler._config_load()
        self.assertEqual("profile_id", self.cahandler.profile_id)

    def test_028_auth_set(self):
        """test _auth_set"""
        self.cahandler.api_user = "api_user"
        self.cahandler.api_password = "api_password"
        self.cahandler._auth_set()
        self.assertTrue(self.cahandler.auth)

    def test_029_auth_set(self):
        """test _auth_set without api_user"""
        self.cahandler.api_user = None
        self.cahandler.api_password = "api_password"
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._auth_set()
        self.assertFalse(self.cahandler.auth)
        self.assertIn(
            'ERROR:test_a2c:Auth information incomplete. Either "api_user" or "api_password" parameter is missing in config file',
            lcm.output,
        )

    def test_030_auth_set(self):
        """test _auth_set without api_user"""
        self.cahandler.api_user = "api_user"
        self.cahandler.api_password = None
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._auth_set()
        self.assertFalse(self.cahandler.auth)
        self.assertIn(
            'ERROR:test_a2c:Auth information incomplete. Either "api_user" or "api_password" parameter is missing in config file',
            lcm.output,
        )

    @patch.object(requests, "post")
    def test_031__api_post(self, mock_req):
        """test _api_post successful run"""
        mockresponse = Mock()
        mock_req.return_value = mockresponse
        mockresponse.json = lambda: {"foo": "bar"}
        self.assertEqual({"foo": "bar"}, self.cahandler._api_post("url", "data"))

    @patch("requests.post")
    def test_032__api_post(self, mock_post):
        """CAhandler.get_ca() returns an http error"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_post.side_effect = Exception("exc_api_post")
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual("exc_api_post", self.cahandler._api_post("url", "data"))
        self.assertIn(
            "ERROR:test_a2c:API post() request returned an error: exc_api_post",
            lcm.output,
        )

    @patch.object(requests, "get")
    def test_033__ca_get(self, mock_req):
        """test _ca_get successful run"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mockresponse = Mock()
        mock_req.return_value = mockresponse
        mockresponse.json = lambda: {"foo": "bar"}
        self.assertEqual({"foo": "bar"}, self.cahandler._ca_get())

    def test_034__api_post(self):
        """test _ca_get no api_host"""
        self.cahandler.auth = "auth"
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual({}, self.cahandler._ca_get())
        self.assertIn(
            "ERROR:test_a2c:api_host parameter is misisng in configuration",
            lcm.output,
        )

    @patch.object(requests, "get")
    def test_035__ca_get(self, mock_req):
        """test _ca_get auth none"""
        self.cahandler.api_host = "api_host"
        mockresponse = Mock()
        mock_req.return_value = mockresponse
        mockresponse.json = lambda: {"foo": "bar"}
        self.assertEqual({"foo": "bar"}, self.cahandler._ca_get())

    @patch("requests.get")
    def test_036__api_post(self, mock_get):
        """CAhandler.get_ca() returns an http error"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_get.side_effect = Exception("exc_ca_get")
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                {
                    "status": 500,
                    "message": "exc_ca_get",
                    "statusMessage": "Internal Server Error",
                },
                self.cahandler._ca_get(),
            )
        self.assertIn(
            "ERROR:test_a2c:API get() request returned error: exc_ca_get", lcm.output
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_037__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns nothing"""
        mock_caget.return_value = []
        self.assertEqual(
            {"status": 404, "message": "CA not found", "statusMessage": "Not Found"},
            self.cahandler._ca_get_properties("filterkey", "filtervalue"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_038__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns wrong information"""
        mock_caget.return_value = "foo"
        self.assertEqual(
            {"status": 404, "message": "CA not found", "statusMessage": "Not Found"},
            self.cahandler._ca_get_properties("filterkey", "filtervalue"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_039__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns error message"""
        mock_caget.return_value = {"status": "status", "message": "message"}
        self.assertEqual(
            {"message": "message", "status": "status"},
            self.cahandler._ca_get_properties("filterkey", "filtervalue"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_040__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns empty ca_list"""
        mock_caget.return_value = {"cas": None}
        self.assertEqual(
            {"status": 404, "message": "CA not found", "statusMessage": "Not Found"},
            self.cahandler._ca_get_properties("filterkey", "filtervalue"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_041__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns ca_list but filter does not match"""
        mock_caget.return_value = {"cas": [{"foo": "bar"}]}
        self.assertEqual(
            {"status": 404, "message": "CA not found", "statusMessage": "Not Found"},
            self.cahandler._ca_get_properties("filterkey", "filtervalue"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_042__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns ca_list but filter matches"""
        mock_caget.return_value = {
            "cas": [{"foo": "bar"}, {"filterkey": "filtervalue"}, {"foo1": "bar1"}]
        }
        self.assertEqual(
            {"filterkey": "filtervalue"},
            self.cahandler._ca_get_properties("filterkey", "filtervalue"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_043__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns ca_list another filterkey"""
        mock_caget.return_value = {
            "cas": [{"foo": "bar"}, {"filterkey": "filtervalue"}, {"foo1": "bar1"}]
        }
        self.assertEqual(
            {"foo": "bar"}, self.cahandler._ca_get_properties("foo", "bar")
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get")
    def test_044__ca_get_properties(self, mock_caget):
        """CAhandler._ca_get_properties() ca_get returns ca_list filterkey check first match"""
        mock_caget.return_value = {
            "cas": [
                {"foo": "bar_bogus"},
                {"foo": "bar"},
                {"foo": "bar1"},
                {"foo": "bar2"},
            ]
        }
        self.assertEqual(
            {"foo": "bar"}, self.cahandler._ca_get_properties("foo", "bar")
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_045__cert_get(self, mock_caget):
        """CAhandler._ca_get_properties() _ca_get_properties returns empty dic"""
        mock_caget.return_value = {}
        self.assertEqual({}, self.cahandler._cert_get("csr"))

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_046__cert_get(self, mock_caget, mock_post):
        """CAhandler._ca_get_properties() _ca_get_properties does returns "href" key"""
        self.cahandler.api_host = "api_host"
        mock_caget.return_value = {"href": "href"}
        mock_post.return_value = {"mock": "post"}
        self.assertEqual({"mock": "post"}, self.cahandler._cert_get("csr"))

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_047__cert_get(self, mock_caget, mock_post):
        """CAhandler._ca_get_properties() _ca_get_properties returns "href" key but cert_dic is empty"""
        self.cahandler.api_host = "api_host"
        mock_caget.return_value = {"href": "href"}
        mock_post.return_value = {}
        self.assertEqual({"href": "href"}, self.cahandler._cert_get("csr"))

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_048__cert_get(self, mock_caget, mock_post):
        """CAhandler._ca_get_properties() _ca_get_properties does returns "href" key"""
        self.cahandler.api_host = "api_host"
        self.cahandler.profile_id = 100
        mock_caget.return_value = {"href": "href"}
        mock_post.return_value = {"mock": "post"}
        self.assertEqual({"mock": "post"}, self.cahandler._cert_get("csr"))

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_049__cert_get(self, mock_caget, mock_post):
        """CAhandler._ca_get_properties() _ca_get_properties does returns "href" key"""
        self.cahandler.api_host = "api_host"
        self.cahandler.profile_id = 100
        self.cahandler.header_info_field = "header_info_field"
        mock_caget.return_value = {"href": "href"}
        mock_post.return_value = {"mock": "post"}
        self.assertEqual({"mock": "post"}, self.cahandler._cert_get("csr"))
        self.assertEqual(100, self.cahandler.profile_id)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.enrollment_config_log")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_050__cert_get(self, mock_caget, mock_post, mock_ecl):
        """CAhandler._ca_get_properties() _ca_get_properties does returns "href" key"""
        self.cahandler.api_host = "api_host"
        mock_caget.return_value = {"href": "href"}
        mock_post.return_value = {"mock": "post"}
        self.assertEqual({"mock": "post"}, self.cahandler._cert_get("csr"))
        self.assertFalse(mock_ecl.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.enrollment_config_log")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_051__cert_get(self, mock_caget, mock_post, mock_ecl):
        """CAhandler._ca_get_properties() _ca_get_properties does returns "href" key"""
        self.cahandler.api_host = "api_host"
        self.cahandler.enrollment_config_log = True
        mock_caget.return_value = {"href": "href"}
        mock_post.return_value = {"mock": "post"}
        self.assertEqual({"mock": "post"}, self.cahandler._cert_get("csr"))
        self.assertTrue(mock_ecl.called)

    @patch("requests.get")
    def test_052__cert_get_properties(self, mock_req):
        """CAhandler._cert_get_properties() all good"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mockresponse = Mock()
        mock_req.return_value = mockresponse
        mockresponse.json = lambda: {"foo": "bar"}
        self.assertEqual(
            {"foo": "bar"}, self.cahandler._cert_get_properties("serial", "link")
        )

    @patch("requests.get")
    def test_053__cert_get_properties(self, mock_get):
        """CAhandler._cert_get_properties() all good"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_get.side_effect = Exception("exc_api_get")
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                {
                    "status": 500,
                    "message": "exc_api_get",
                    "statusMessage": "Internal Server Error",
                },
                self.cahandler._cert_get_properties("serial", "link"),
            )
        self.assertIn(
            "ERROR:test_a2c:Could not get certificate properties. Error: exc_api_get",
            lcm.output,
        )

    def test_054_poll(self):
        """CAhandler.poll() poll_identifier is none"""
        self.assertEqual(
            (None, None, None, None, False),
            self.cahandler.poll("cert_name", None, "csr"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._request_poll")
    def test_055_poll(self, mock_poll):
        """CAhandler.poll() poll_identifier is none"""
        mock_poll.return_value = (
            "error",
            "cert_bundle",
            "cert_raw",
            "poll_identifier",
            "rejected",
        )
        self.assertEqual(
            ("error", "cert_bundle", "cert_raw", "poll_identifier", "rejected"),
            self.cahandler.poll("cert_name", "poll_identifier", "csr"),
        )

    def test_056__loop_poll(self):
        """CAhandler._loop_poll() - no request url"""
        request_url = None
        self.assertEqual(
            (None, None, None, None), self.cahandler._loop_poll(request_url)
        )

    @patch("time.sleep")
    @patch("requests.get")
    def test_057__loop_poll(self, mock_get, mock_sleep):
        """CAhandler._loop_poll() - nothing come back from request get"""
        self.cahandler.polling_timeout = 5
        self.cahandler.timeout = 0
        request_url = "request_url"
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mock_sleep.return_value = mockresponse
        mockresponse.json = lambda: {}
        self.assertEqual(
            (None, None, None, "request_url"), self.cahandler._loop_poll(request_url)
        )

    @patch("time.sleep")
    @patch("requests.get")
    def test_058__loop_poll(self, mock_get, mock_sleep):
        """CAhandler._loop_poll() - no status returned from  request get"""
        self.cahandler.polling_timeout = 5
        self.cahandler.timeout = 0
        request_url = "request_url"
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mock_sleep.return_value = mockresponse
        mockresponse.json = lambda: {"foo": "bar"}
        self.assertEqual(
            (None, None, None, "request_url"), self.cahandler._loop_poll(request_url)
        )

    @patch("requests.get")
    def test_059__loop_poll(self, mock_get):
        """CAhandler._loop_poll() - status "rejected" returned from  request get"""
        self.cahandler.polling_timeout = 6
        self.cahandler.timeout = 0
        request_url = "request_url"
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mockresponse.json = lambda: {"status": "rejected", "foo": "bar"}
        self.assertEqual(
            ("Request rejected by operator", None, None, None),
            self.cahandler._loop_poll(request_url),
        )

    @patch("time.sleep")
    @patch("requests.get")
    def test_060__loop_poll(self, mock_get, mock_sleep):
        """CAhandler._loop_poll() - status "accepted" returned from  request get but no certificate in"""
        self.cahandler.polling_timeout = 6
        self.cahandler.timeout = 0
        request_url = "request_url"
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mock_sleep.return_value = mockresponse
        mockresponse.json = lambda: {"status": "accepted", "foo": "bar"}
        self.assertEqual(
            ("Request accepted but no certificate returned", None, None, "request_url"),
            self.cahandler._loop_poll(request_url),
        )

    @patch("time.sleep")
    @patch("requests.get")
    def test_061__loop_poll(self, mock_get, mock_sleep):
        """CAhandler._loop_poll() - status "accepted" returned from  request "certifiate" in but no "certificateBase64" in 2dn request"""
        self.cahandler.polling_timeout = 6
        self.cahandler.timeout = 0
        request_url = "request_url"
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mock_sleep.return_value = mockresponse
        mockresponse.json = lambda: {
            "status": "accepted",
            "foo": "bar",
            "certificate": "certificate",
        }
        self.assertEqual(
            (
                "Request accepted but no certificateBase64 returned",
                None,
                None,
                "request_url",
            ),
            self.cahandler._loop_poll(request_url),
        )

    @patch(
        "acme2certifier.cahandlers.certifier_ca_handler.CAhandler._pem_cert_chain_generate"
    )
    @patch("requests.get")
    def test_062__loop_poll(self, mock_get, mock_chain):
        """CAhandler._loop_poll() - status "accepted" returned from  request "certifiate" in but no "certificateBase64" in 2dn request"""
        self.cahandler.polling_timeout = 6
        self.cahandler.timeout = 0
        request_url = "request_url"
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mockresponse.json = lambda: {
            "status": "accepted",
            "foo": "bar",
            "certificate": "certificate",
            "certificateBase64": "certificateBase64",
        }
        mock_chain.return_value = "foo"
        self.assertEqual(
            (None, "foo", "certificateBase64", None),
            self.cahandler._loop_poll(request_url),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_063_enroll(self, mock_certget):
        """CAhandler.enroll() _cert_get returns None"""
        mock_certget.return_value = {}
        self.assertEqual(
            ("internal error", None, None, None), self.cahandler.enroll("csr")
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_064_enroll(self, mock_certget):
        """CAhandler.enroll() _cert_get returns wrong information"""
        mock_certget.return_value = {"foo": "bar"}
        self.assertEqual(
            ("no certificate information found", None, None, None),
            self.cahandler.enroll("csr"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_065_enroll(self, mock_certget):
        """CAhandler.enroll() _cert_get returns status without error message"""
        mock_certget.return_value = {"foo": "bar", "status": "foo"}
        self.assertEqual(
            ("unknown error", None, None, None), self.cahandler.enroll("csr")
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_066_enroll(self, mock_certget):
        """CAhandler.enroll() _cert_get returns status with error message"""
        mock_certget.return_value = {
            "foo": "bar",
            "status": "foo",
            "message": "message",
        }
        self.assertEqual(("message", None, None, None), self.cahandler.enroll("csr"))

    @patch(
        "acme2certifier.cahandlers.certifier_ca_handler.CAhandler._pem_cert_chain_generate"
    )
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_067_enroll(self, mock_certget, mock_chain):
        """CAhandler.enroll() _cert_get returns certb64"""
        mock_certget.return_value = {
            "foo": "bar",
            "certificateBase64": "certificateBase64",
        }
        mock_chain.return_value = "mock_chain"
        self.assertEqual(
            (None, "mock_chain", "certificateBase64", None),
            self.cahandler.enroll("csr"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._loop_poll")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_068_enroll(self, mock_certget, mock_loop):
        """CAhandler.enroll() _cert_get returns certb64"""
        mock_certget.return_value = {"foo": "bar", "href": "href"}
        mock_loop.return_value = ("error", "cert_bundle", "cert_raw", "poll_identifier")
        self.assertEqual(
            ("error", "cert_bundle", "cert_raw", "poll_identifier"),
            self.cahandler.enroll("csr"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_header_info_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._loop_poll")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_069_enroll(self, mock_certget, mock_loop, mock_prof):
        """CAhandler.enroll() _cert_get returns certb64"""
        mock_certget.return_value = {"foo": "bar", "href": "href"}
        mock_loop.return_value = ("error", "cert_bundle", "cert_raw", "poll_identifier")
        mock_prof.return_value = None
        self.assertEqual(
            ("error", "cert_bundle", "cert_raw", "poll_identifier"),
            self.cahandler.enroll("csr"),
        )
        self.assertTrue(mock_prof.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_header_info_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._loop_poll")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_070_enroll(self, mock_certget, mock_loop, mock_prof):
        """CAhandler.enroll() _cert_get returns certb64"""
        mock_certget.return_value = {"foo": "bar", "href": "href"}
        mock_loop.return_value = ("error", "cert_bundle", "cert_raw", "poll_identifier")
        mock_prof.return_value = None
        self.cahandler.eab_profiling = True
        self.cahandler.header_info_field = "header_info_field"
        self.assertEqual(
            ("error", "cert_bundle", "cert_raw", "poll_identifier"),
            self.cahandler.enroll("csr"),
        )
        self.assertTrue(mock_prof.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_header_info_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._loop_poll")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_071_enroll(self, mock_certget, mock_loop, mock_prof):
        """CAhandler.enroll() _cert_get returns certb64"""
        mock_certget.return_value = {"foo": "bar", "href": "href"}
        mock_loop.return_value = ("error", "cert_bundle", "cert_raw", "poll_identifier")
        self.cahandler.eab_profiling = True
        self.cahandler.header_info_field = "header_info_field"
        mock_prof.return_value = "prof_error"
        self.assertEqual(("prof_error", None, None, None), self.cahandler.enroll("csr"))
        self.assertTrue(mock_prof.called)
        self.assertFalse(mock_certget.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_header_info_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._loop_poll")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get")
    def test_072_enroll(self, mock_certget, mock_loop, mock_prof):
        """CAhandler.enroll() _cert_get returns certb64"""
        mock_certget.return_value = {"foo": "bar", "href": "href"}
        mock_loop.return_value = ("error", "cert_bundle", "cert_raw", "poll_identifier")
        self.cahandler.eab_profiling = False
        self.cahandler.header_info_field = "header_info_field"
        mock_prof.return_value = None
        self.assertEqual(
            ("error", "cert_bundle", "cert_raw", "poll_identifier"),
            self.cahandler.enroll("csr"),
        )
        self.assertTrue(mock_prof.called)
        self.assertTrue(mock_certget.called)
        self.assertEqual(self.cahandler.profile_id, None)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_073_revoke(self, mock_getca):
        """CAhandler.revoke() _ca_get_properties returns nothing"""
        mock_getca.return_value = {}
        self.assertEqual(
            (404, "urn:ietf:params:acme:error:serverInternal", "CA could not be found"),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_074_revoke(self, mock_getca):
        """CAhandler.revoke() _ca_get_properties returns wrong information"""
        mock_getca.return_value = {"foo": "bar"}
        self.assertEqual(
            (404, "urn:ietf:params:acme:error:serverInternal", "CA could not be found"),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_075_revoke(self, mock_getca, mock_serial):
        """CAhandler.revoke() _ca_get_properties cert_serial_get failed"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = None
        self.assertEqual(
            (
                404,
                "urn:ietf:params:acme:error:serverInternal",
                "failed to get serial number from cert",
            ),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_076_revoke(self, mock_getca, mock_serial, mock_getcert):
        """CAhandler.revoke() _ca_get_properties get_cert_properties failed"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {}
        self.assertEqual(
            (
                404,
                "urn:ietf:params:acme:error:serverInternal",
                "Cert could not be found",
            ),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_077_revoke(self, mock_getca, mock_serial, mock_getcert):
        """CAhandler.revoke() _ca_get_properties get_cert_properties returns wrong information"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {"foo": "bar"}
        self.assertEqual(
            (
                404,
                "urn:ietf:params:acme:error:serverInternal",
                "Cert could not be found",
            ),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_078_revoke(self, mock_getca, mock_serial, mock_getcert):
        """CAhandler.revoke() _ca_get_properties get_cert_properties empty cert_list"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {"foo": "bar", "certificates": []}
        self.assertEqual(
            (
                404,
                "urn:ietf:params:acme:error:serverInternal",
                "Cert path could not be found",
            ),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_079_revoke(self, mock_getca, mock_serial, mock_getcert):
        """CAhandler.revoke() _ca_get_properties get_cert_properties returns cert_list with wrong information"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {"foo": "bar", "certificates": [{"foo": "bar"}]}
        self.assertEqual(
            (
                404,
                "urn:ietf:params:acme:error:serverInternal",
                "Cert path could not be found",
            ),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_revocation_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_080_revoke(
        self, mock_getca, mock_serial, mock_getcert, mock_post, mock_eab
    ):
        """CAhandler.revoke() _ca_get_properties get_cert_properties returns cert_list revocation successful"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {
            "foo": "bar",
            "certificates": [{"foo": "bar", "href": "href"}],
        }
        mock_post.return_value = {}
        self.assertEqual((200, None, None), self.cahandler.revoke("cert"))
        self.assertFalse(mock_eab.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_revocation_check")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_081_revoke(
        self, mock_getca, mock_serial, mock_getcert, mock_post, mock_eab
    ):
        """CAhandler.revoke() _ca_get_properties get_cert_properties returns cert_list revocation successful"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        self.cahandler.eab_profiling = True
        mock_serial.return_value = 123
        mock_getcert.return_value = {
            "foo": "bar",
            "certificates": [{"foo": "bar", "href": "href"}],
        }
        mock_post.return_value = {}
        self.assertEqual((200, None, None), self.cahandler.revoke("cert"))
        self.assertTrue(mock_eab.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_082_revoke(self, mock_getca, mock_serial, mock_getcert, mock_post):
        """CAhandler.revoke() _ca_get_properties get_cert_properties returns href. revocation returns status without message"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {
            "foo": "bar",
            "certificates": [{"foo": "bar", "href": "href"}],
        }
        mock_post.return_value = {"foo": "bar", "status": "status"}
        self.assertEqual(
            (400, "urn:ietf:params:acme:error:alreadyRevoked", "no details"),
            self.cahandler.revoke("cert"),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    def test_083_revoke(self, mock_getca, mock_serial, mock_getcert, mock_post):
        """CAhandler.revoke() _ca_get_properties get_cert_properties returns href. revocation returns status with message"""
        mock_getca.return_value = {"foo": "bar", "href": "href"}
        mock_serial.return_value = 123
        mock_getcert.return_value = {
            "foo": "bar",
            "certificates": [{"foo": "bar", "href": "href"}],
        }
        mock_post.return_value = {
            "foo": "bar",
            "status": "status",
            "message": "message",
        }
        self.assertEqual(
            (400, "urn:ietf:params:acme:error:alreadyRevoked", "message"),
            self.cahandler.revoke("cert"),
        )

    def test_084_trigger(self):
        """CAhandler.trigger() - no payload given"""
        payload = None
        self.assertEqual(
            ("No payload given", None, None), self.cahandler.trigger(payload)
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_pem2der")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_encode")
    def test_085_trigger(self, mock_b64dec, mock_b64enc, mock_p2d, mock_caprop):
        """CAhandler.trigger() - payload  but ca_lookup failed"""
        payload = "foo"
        mock_b64dec.return_value = "foodecode"
        mock_p2d.return_value = "p2d"
        mock_caprop.return_value = {}
        self.assertEqual(
            ("Cannot find CA", None, "foodecode"), self.cahandler.trigger(payload)
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_pem2der")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_encode")
    def test_086_trigger(
        self, mock_b64dec, mock_b64enc, mock_p2d, mock_caprop, mock_serial
    ):
        """CAhandler.trigger() - payload serial number lookup failed"""
        payload = "foo"
        mock_b64dec.return_value = "foodecode"
        mock_serial.return_value = None
        mock_p2d.return_value = "p2d"
        mock_caprop.return_value = {"href": "href"}
        self.assertEqual(
            ("serial number lookup via rest failed", None, "foodecode"),
            self.cahandler.trigger(payload),
        )

    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_pem2der")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_encode")
    def test_087_trigger(
        self,
        mock_b64dec,
        mock_b64enc,
        mock_p2d,
        mock_caprop,
        mock_serial,
        mock_certprop,
    ):
        """CAhandler.trigger() - payload serial number lookup failed"""
        payload = "foo"
        mock_b64dec.return_value = "foodecode"
        mock_serial.return_value = 123
        mock_p2d.return_value = "p2d"
        mock_caprop.return_value = {"href": "href"}
        mock_certprop.return_value = {}
        self.assertEqual(
            ("no certifcates found in rest query", None, "foodecode"),
            self.cahandler.trigger(payload),
        )

    @patch(
        "acme2certifier.cahandlers.certifier_ca_handler.CAhandler._pem_cert_chain_generate"
    )
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_pem2der")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_encode")
    def test_088_trigger(
        self,
        mock_b64dec,
        mock_b64enc,
        mock_p2d,
        mock_caprop,
        mock_serial,
        mock_certprop,
        mock_chain,
    ):
        """CAhandler.trigger() - payload serial number lookup failed"""
        payload = "foo"
        mock_b64dec.return_value = "foodecode"
        mock_serial.return_value = 123
        mock_p2d.return_value = "p2d"
        mock_caprop.return_value = {"href": "href"}
        mock_certprop.return_value = {"certificates": [{"foo": "bar"}]}
        mock_chain.return_value = "chain"
        self.assertEqual((None, "chain", "foodecode"), self.cahandler.trigger(payload))

    @patch(
        "acme2certifier.cahandlers.certifier_ca_handler.CAhandler._pem_cert_chain_generate"
    )
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._cert_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_serial_get")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.CAhandler._ca_get_properties")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.cert_pem2der")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.certifier_ca_handler.b64_encode")
    def test_089_trigger(
        self,
        mock_b64dec,
        mock_b64enc,
        mock_p2d,
        mock_caprop,
        mock_serial,
        mock_certprop,
        mock_chain,
    ):
        """CAhandler.trigger() - payload serial number lookup failed"""
        payload = "foo"
        mock_b64dec.return_value = "foodecode"
        mock_serial.return_value = 123
        mock_p2d.side_effect = Exception("p2d")
        mock_caprop.return_value = {"href": "href"}
        mock_certprop.return_value = {"certificates": [{"foo": "bar"}]}
        mock_chain.return_value = "chain"
        self.assertEqual((None, "chain", "foodecode"), self.cahandler.trigger(payload))

    def test_090__pem_cert_chain_generate(self):
        """_pem_cert_chain_generate - empty cert_dic"""
        cert_dic = {}
        self.assertFalse(self.cahandler._pem_cert_chain_generate(cert_dic))

    def test_091__pem_cert_chain_generate(self):
        """_pem_cert_chain_generate - wrong dic"""
        cert_dic = {"foo": "bar"}
        self.assertFalse(self.cahandler._pem_cert_chain_generate(cert_dic))

    def test_092__pem_cert_chain_generate(self):
        """_pem_cert_chain_generate - certificateBase64 in dict"""
        cert_dic = {"certificateBase64": "certificateBase64"}
        self.assertEqual(
            "-----BEGIN CERTIFICATE-----\ncertificateBase64\n-----END CERTIFICATE-----\n",
            self.cahandler._pem_cert_chain_generate(cert_dic),
        )

    @patch("requests.get")
    def test_093__pem_cert_chain_generate(self, mock_get):
        """_pem_cert_chain_generate - issuer in dict without certificateBase64"""
        cert_dic = {"issuer": "issuer"}
        mockresponse = Mock()
        mock_get.return_value = mockresponse
        mockresponse.json = lambda: {"foo": "bar"}
        self.assertFalse(self.cahandler._pem_cert_chain_generate(cert_dic))

    @patch("requests.get")
    def test_094__pem_cert_chain_generate(self, mock_get):
        """_pem_cert_chain_generate - request returns "certificates" but no active"""
        cert_dic = {"issuer": "issuer", "certificateBase64": "certificateBase641"}
        mockresponse1 = Mock()
        mockresponse1.json = lambda: {"certificates": "certificates"}
        mockresponse2 = Mock()
        mockresponse2.json = lambda: {"foo": "bar"}
        mock_get.side_effect = [mockresponse1, mockresponse2]
        self.assertEqual(
            "-----BEGIN CERTIFICATE-----\ncertificateBase641\n-----END CERTIFICATE-----\n",
            self.cahandler._pem_cert_chain_generate(cert_dic),
        )

    @patch("requests.get")
    def test_095__pem_cert_chain_generate(self, mock_get):
        """_pem_cert_chain_generate - request returns certificate and active, 2nd request is bogus"""
        cert_dic = {"issuer": "issuer", "certificateBase64": "certificateBase641"}
        mockresponse1 = Mock()
        mockresponse1.json = lambda: {"certificates": {"active": "active"}}
        mockresponse2 = Mock()
        mockresponse2.json = lambda: {"foo": "bar"}
        mock_get.side_effect = [mockresponse1, mockresponse2]
        self.assertEqual(
            "-----BEGIN CERTIFICATE-----\ncertificateBase641\n-----END CERTIFICATE-----\n",
            self.cahandler._pem_cert_chain_generate(cert_dic),
        )

    @patch("requests.get")
    def test_096__pem_cert_chain_generate(self, mock_get):
        """_pem_cert_chain_generate - request returns certificate two certs"""
        cert_dic = {"issuer": "issuer", "certificateBase64": "certificateBase641"}
        mockresponse1 = Mock()
        mockresponse1.json = lambda: {"certificates": {"active": "active"}}
        mockresponse2 = Mock()
        mockresponse2.json = lambda: {
            "certificateBase64": "certificateBase642",
            "issuer": "issuer",
        }
        mockresponse3 = Mock()
        mockresponse3.json = lambda: {"foo": "bar"}
        mock_get.side_effect = [mockresponse1, mockresponse2, mockresponse3]
        self.assertEqual(
            "-----BEGIN CERTIFICATE-----\ncertificateBase641\n-----END CERTIFICATE-----\n-----BEGIN CERTIFICATE-----\ncertificateBase642\n-----END CERTIFICATE-----\n",
            self.cahandler._pem_cert_chain_generate(cert_dic),
        )

    @patch("requests.get")
    def test_097__pem_cert_chain_generate(self, mock_get):
        """_pem_cert_chain_generate - request returns certificate three certs"""
        cert_dic = {"issuer": "issuer", "certificateBase64": "certificateBase641"}
        mockresponse1 = Mock()
        mockresponse1.json = lambda: {"certificates": {"active": "active"}}
        mockresponse2 = Mock()
        mockresponse2.json = lambda: {
            "certificateBase64": "certificateBase642",
            "issuer": "issuer",
        }
        mockresponse3 = Mock()
        mockresponse3.json = lambda: {"certificates": {"active": "active"}}
        mockresponse4 = Mock()
        mockresponse4.json = lambda: {
            "certificateBase64": "certificateBase643",
            "issuer": "issuer",
        }
        mockresponse5 = Mock()
        mockresponse5.json = lambda: {"foo": "bar"}
        mock_get.side_effect = [
            mockresponse1,
            mockresponse2,
            mockresponse3,
            mockresponse4,
            mockresponse5,
        ]
        self.assertEqual(
            "-----BEGIN CERTIFICATE-----\ncertificateBase641\n-----END CERTIFICATE-----\n-----BEGIN CERTIFICATE-----\ncertificateBase642\n-----END CERTIFICATE-----\n-----BEGIN CERTIFICATE-----\ncertificateBase643\n-----END CERTIFICATE-----\n",
            self.cahandler._pem_cert_chain_generate(cert_dic),
        )

    @patch("requests.get")
    def test_098__pem_cert_chain_generate(self, mock_get):
        """_pem_cert_chain_generate - issuerCa in"""
        cert_dic = {"issuerCa": "issuerCa", "certificateBase64": "certificateBase641"}
        mockresponse1 = Mock()
        mockresponse1.json = lambda: {"certificates": "certificates"}
        mockresponse2 = Mock()
        mockresponse2.json = lambda: {"foo": "bar"}
        mock_get.side_effect = [mockresponse1, mockresponse2]
        self.assertEqual(
            "-----BEGIN CERTIFICATE-----\ncertificateBase641\n-----END CERTIFICATE-----\n",
            self.cahandler._pem_cert_chain_generate(cert_dic),
        )

    def test_099__enter__(self):
        """test __enter__"""
        self.cahandler.__enter__()

    @patch("requests.get")
    def test_100_request_poll(self, mock_get):
        """test request poll request returned exception"""
        mock_get.side_effect = Exception("exc_api_get")
        result = ('"status" field not found in response.', None, None, "url", False)
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(result, self.cahandler._request_poll("url"))
        self.assertIn(
            "ERROR:test_a2c:Polling request returned an error: exc_api_get", lcm.output
        )

    @patch("requests.get")
    def test_101_request_poll(self, mock_get):
        """test request poll request returned unknown status"""
        mockresponse = Mock()
        mockresponse.json = lambda: {"status": "unknown"}
        mock_get.return_value = mockresponse
        result = ("Unknown request status: unknown", None, None, "url", False)
        self.assertEqual(result, self.cahandler._request_poll("url"))

    @patch("requests.get")
    def test_102_request_poll(self, mock_get):
        """test request poll request returned status rejected"""
        mockresponse = Mock()
        mockresponse.json = lambda: {"status": "rejected"}
        mock_get.return_value = mockresponse
        result = ("Request rejected by operator", None, None, "url", True)
        self.assertEqual(result, self.cahandler._request_poll("url"))

    @patch("requests.get")
    def test_103_request_poll(self, mock_get):
        """test request poll request returned status accepted but no certinformation in"""
        mockresponse = Mock()
        mockresponse.json = lambda: {"status": "accepted", "foo": "bar"}
        mock_get.return_value = mockresponse
        result = (
            "No certificate structure in request response",
            None,
            None,
            "url",
            False,
        )
        self.assertEqual(result, self.cahandler._request_poll("url"))

    @patch("requests.get")
    def test_104_request_poll(self, mock_get):
        """test request poll request returned status accepted but no certinformation in"""
        mockresponse = Mock()
        mockresponse.json = lambda: {"status": "accepted", "certificate": "certificate"}
        mock_get.return_value = mockresponse
        result = (
            "certificateBase64 is missing in cert request response",
            None,
            None,
            "url",
            False,
        )
        self.assertEqual(result, self.cahandler._request_poll("url"))

    @patch(
        "acme2certifier.cahandlers.certifier_ca_handler.CAhandler._pem_cert_chain_generate"
    )
    @patch("requests.get")
    def test_105_request_poll(self, mock_get, mock_pemgen):
        """test request poll request returned status accepted but no certinformation in"""
        mockresponse = Mock()
        mockresponse.json = lambda: {
            "status": "accepted",
            "certificate": "certificate",
            "certificateBase64": "certificateBase64",
        }
        mock_get.return_value = mockresponse
        mock_pemgen.return_value = "bundle"
        result = (None, "bundle", "certificateBase64", "url", False)
        self.assertEqual(result, self.cahandler._request_poll("url"))

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_header_info_check")
    def test_106_csr_check(self, mock_eab):
        """test csr_check"""
        csr = "csr"
        mock_eab.return_value = None
        self.assertEqual(None, self.cahandler._csr_check(csr))
        self.assertTrue(mock_eab.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.eab_profile_header_info_check")
    def test_107_csr_check(self, mock_eab):
        """test csr_check"""
        csr = "csr"
        mock_eab.return_value = "mock_eab"
        self.assertEqual("mock_eab", self.cahandler._csr_check(csr))
        self.assertTrue(mock_eab.called)

    @patch("acme2certifier.cahandlers.certifier_ca_handler.handler_config_check")
    def test_108_handler_check(self, mock_handler_check):
        """test handler_check"""
        mock_handler_check.return_value = "mock_handler_check"
        self.assertEqual("mock_handler_check", self.cahandler.handler_check())


if __name__ == "__main__":

    if os.path.exists("acme_test.db"):
        os.remove("acme_test.db")
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""unittests for account.py"""

# pylint: disable=C0302, C0415, R0904, R0913, R0914, R0915, W0212
import unittest
import sys
import importlib
import configparser
from unittest.mock import patch, MagicMock, Mock

sys.path.insert(0, ".")
sys.path.insert(1, "..")


class FakeDBStore(object):
    """face DBStore class needed for mocking"""

    # pylint: disable=W0107, R0903
    pass


class TestACMEHandler(unittest.TestCase):
    """test class for ACMEHandler"""

    acme = None

    def setUp(self):
        """setup unittest"""
        models_mock = MagicMock()
        modules = {"acme2certifier.acme_srv.db_handler": models_mock}
        patch.dict("sys.modules", modules).start()
        import logging

        logging.basicConfig(level=logging.CRITICAL)
        self.logger = logging.getLogger("test_a2c")
        from acme2certifier.acme_srv.trigger import Trigger
        from acme2certifier.acme_srv.order import Order

        self.order = Order(False, "http://tester.local", self.logger)
        self.trigger = Trigger(False, "http://tester.local", self.logger)

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.certificate.Certificate.certlist_search")
    @patch("acme2certifier.acme_srv.trigger.cert_pubkey_get")
    def test_001_trigger__certname_lookup(
        self, mock_cert_pub, mock_search_list, mock_import
    ):
        """trigger._certname_lookup() failed bcs. of empty certificate list"""
        mock_cert_pub.return_value = "foo"
        mock_search_list.return_value = []
        mock_import.return_value = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.assertEqual([], self.trigger._certname_lookup("cert_pem"))

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.certificate.Certificate.certlist_search")
    @patch("acme2certifier.acme_srv.trigger.cert_pubkey_get")
    def test_002_trigger__certname_lookup(
        self, mock_cert_pub, mock_search_list, mock_import
    ):
        """trigger._certname_lookup() failed bcs. of wrong certificate list"""
        mock_cert_pub.return_value = "foo"
        mock_search_list.return_value = [{"foo": "bar"}]
        mock_import.return_value = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.assertEqual([], self.trigger._certname_lookup("cert_pem"))

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.certificate.Certificate.certlist_search")
    @patch("acme2certifier.acme_srv.trigger.cert_pubkey_get")
    def test_003_trigger__certname_lookup(
        self, mock_cert_pub, mock_search_list, mock_import
    ):
        """trigger._certname_lookup() failed bcs. of emty csr field"""
        mock_cert_pub.return_value = "foo"
        mock_search_list.return_value = [{"csr": None}]
        mock_import.return_value = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.assertEqual([], self.trigger._certname_lookup("cert_pem"))

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.trigger.csr_pubkey_get")
    @patch("acme2certifier.acme_srv.certificate.Certificate.certlist_search")
    @patch("acme2certifier.acme_srv.trigger.cert_pubkey_get")
    def test_004_trigger__certname_lookup(
        self, mock_cert_pub, mock_search_list, mock_csr_pub, mock_import
    ):
        """trigger._certname_lookup() failed bcs. of emty csr field"""
        mock_cert_pub.return_value = "foo"
        mock_csr_pub.return_value = "foo1"
        mock_search_list.return_value = [{"csr": None}]
        mock_import.return_value = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.assertEqual([], self.trigger._certname_lookup("cert_pem"))

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.trigger.csr_pubkey_get")
    @patch("acme2certifier.acme_srv.certificate.Certificate.certlist_search")
    @patch("acme2certifier.acme_srv.trigger.cert_pubkey_get")
    def test_005_trigger__certname_lookup(
        self, mock_cert_pub, mock_search_list, mock_csr_pub, mock_import
    ):
        """trigger._certname_lookup() failed bcs. of emty csr field"""
        mock_cert_pub.return_value = "foo"
        mock_csr_pub.return_value = "foo"
        mock_search_list.return_value = [
            {"csr": "csr", "name": "cert_name", "order__name": "order_name"}
        ]
        mock_import.return_value = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.assertEqual(
            [{"cert_name": "cert_name", "order_name": "order_name"}],
            self.trigger._certname_lookup("cert_pem"),
        )

    def test_006_trigger_parse(self):
        """Trigger.parse() with empty payload"""
        payload = ""
        result = {
            "header": {},
            "code": 400,
            "data": {"detail": "payload missing", "type": "malformed", "status": 400},
        }
        self.assertEqual(result, self.trigger.parse(payload))

    def test_007_trigger_parse(self):
        """Trigger.parse() with wrong payload"""
        payload = '{"foo": "bar"}'
        result = {
            "header": {},
            "code": 400,
            "data": {"detail": "payload missing", "type": "malformed", "status": 400},
        }
        self.assertEqual(result, self.trigger.parse(payload))

    def test_008_trigger_parse(self):
        """Trigger.parse() with empty payload key"""
        payload = '{"payload": ""}'
        result = {
            "header": {},
            "code": 400,
            "data": {"detail": "payload empty", "type": "malformed", "status": 400},
        }
        self.assertEqual(result, self.trigger.parse(payload))

    @patch("acme2certifier.acme_srv.trigger.Trigger._payload_process")
    def test_009_trigger_parse(self, mock_process):
        """Trigger.parse() with payload mock result 400"""
        payload = '{"payload": "foo"}'
        mock_process.return_value = (400, "message", "detail")
        result = {
            "header": {},
            "code": 400,
            "data": {"detail": "detail", "type": "message", "status": 400},
        }
        self.assertEqual(result, self.trigger.parse(payload))

    @patch("acme2certifier.acme_srv.trigger.Trigger._payload_process")
    def test_010_trigger_parse(self, mock_process):
        """Trigger.parse() with payload mock result 200"""
        payload = '{"payload": "foo"}'
        mock_process.return_value = (200, "message", "detail")
        result = {
            "header": {},
            "code": 200,
            "data": {"detail": "detail", "type": "message", "status": 200},
        }
        self.assertEqual(result, self.trigger.parse(payload))

    def test_011_trigger__payload_process(self):
        """Trigger._payload_process() without payload"""
        payload = {}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", None, None))
        self.assertEqual(
            (400, "payload malformed", None), self.trigger._payload_process(payload)
        )

    def test_012_trigger__payload_process(self):
        """Trigger._payload_process() without certbunde and cert_raw"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", None, None))
        self.assertEqual((400, "error", None), self.trigger._payload_process(payload))

    def test_013_trigger__payload_process(self):
        """Trigger._payload_process() with bundle and without cart_raw"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", "bundle", None))
        self.assertEqual((400, "error", None), self.trigger._payload_process(payload))

    def test_014_trigger__payload_process(self):
        """Trigger._payload_process() with bundle and without cart_raw"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", None, "raw"))
        self.assertEqual((400, "error", None), self.trigger._payload_process(payload))

    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_015_trigger__payload_process(
        self, mock_cobystr, mock_der2pem, mock_b64dec, mock_lookup
    ):
        """Trigger._payload_process() with certificae_name"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", "bundle", "raw"))
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = [
            {"cert_name": "certificate_name", "order_name": None}
        ]
        self.assertEqual((200, "OK", None), self.trigger._payload_process(payload))

    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_016_trigger__payload_process(
        self, mock_cobystr, mock_der2pem, mock_b64dec, mock_lookup
    ):
        """Trigger._payload_process() without certificate_name"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", "bundle", "raw"))
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = [{"cert_name": None, "order_name": "order_name"}]
        self.assertEqual((200, "OK", None), self.trigger._payload_process(payload))

    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_017_trigger__payload_process(
        self, mock_cobystr, mock_der2pem, mock_b64dec, mock_lookup
    ):
        """Trigger._payload_process() _certname.lookup() returned empty list"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", "bundle", "raw"))
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = []
        self.assertEqual(
            (400, "certificate_name lookup failed", None),
            self.trigger._payload_process(payload),
        )

    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_018_trigger__payload_process(
        self, mock_cobystr, mock_der2pem, mock_b64dec, mock_lookup
    ):
        """Trigger._payload_process() without certificate_name"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", "bundle", "raw"))
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = [
            {"cert_name": "certificate_name", "order_name": "order_name"}
        ]
        self.order.dbstore.order_update.return_value = None
        self.assertEqual((200, "OK", None), self.trigger._payload_process(payload))

    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_019_trigger__payload_process(
        self, mock_cobystr, mock_der2pem, mock_b64dec, mock_lookup
    ):
        """Trigger._payload_process() without certificate_name"""
        payload = {"payload": "foo"}
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(return_value=("error", "bundle", "raw"))
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = [
            {"cert_name": "certificate_name1", "order_name": "order_name1"},
            {"cert_name": "certificate_name2", "order_name": "order_name2"},
        ]
        self.assertEqual((200, "OK", None), self.trigger._payload_process(payload))

    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_020_trigger__payload_process(
        self, mock_cobystr, mock_lookup, mock_der2pem, mock_b64dec
    ):
        """test Trigger._payload_process - dbstore.order_update() raises an exception"""
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(
            return_value=(None, "certificate", "certificate_raw")
        )
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = [
            {"cert_name": "certificate_name1", "order_name": "order_name1"},
            {"cert_name": "certificate_name2", "order_name": "order_name2"},
        ]
        self.trigger.dbstore.certificate_add.return_value = True
        self.trigger.dbstore.order_update.side_effect = Exception(
            "exc_trigger_order_upd"
        )
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._payload_process("payload")
        self.assertIn(
            "CRITICAL:test_a2c:Database error: failed to update order status during trigger processing: exc_trigger_order_upd",
            lcm.output,
        )

    @patch("acme2certifier.acme_srv.trigger.b64_decode")
    @patch("acme2certifier.acme_srv.trigger.cert_der2pem")
    @patch("acme2certifier.acme_srv.trigger.Trigger._certname_lookup")
    @patch("acme2certifier.acme_srv.trigger.convert_byte_to_string")
    def test_021_trigger__payload_process(
        self, mock_cobystr, mock_lookup, mock_der2pem, mock_b64dec
    ):
        """test Trigger._payload_process - dbstore.order_update() raises an exception"""
        ca_handler_module = importlib.import_module(
            "acme2certifier.cahandlers.skeleton_ca_handler"
        )
        self.trigger.cahandler = ca_handler_module.CAhandler
        self.trigger.cahandler.trigger = Mock(
            return_value=(None, "certificate", "certificate_raw")
        )
        mock_der2pem.return_value = "der2pem"
        mock_cobystr.return_value = "cert_pem"
        mock_b64dec.return_value = "b64dec"
        mock_lookup.return_value = [
            {"cert_name": "certificate_name1", "order_name": "order_name1"},
            {"cert_name": "certificate_name2", "order_name": "order_name2"},
        ]
        self.trigger.dbstore.certificate_add.side_effect = Exception(
            "exc_trigger_order_add"
        )
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._payload_process("payload")
        self.assertIn(
            "CRITICAL:test_a2c:Database error: failed to update order status during trigger processing: exc_trigger_order_upd",
            lcm.output,
        )

    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_022_config_load(self, mock_load_cfg):
        """test _config_load missing ca_handler"""
        mock_load_cfg.return_value = {}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._config_load()
        self.assertIn(
            "ERROR:test_a2c:CAhandler configuration missing in config file", lcm.output
        )

    @patch("acme2certifier.acme_srv.trigger.Trigger._config_load")
    def test_023__enter__(self, mock_cfg):
        """test enter"""
        mock_cfg.return_value = True
        self.trigger.__enter__()
        self.assertTrue(mock_cfg.called)

    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_024_config_load(self, mock_load_cfg):
        """test _config_load empty config"""
        parser = configparser.ConfigParser()
        # parser['Account'] = {'foo': 'bar'}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._config_load()
        self.assertFalse(self.trigger.tnauthlist_support)
        self.assertIn(
            "ERROR:test_a2c:CAhandler configuration missing in config file", lcm.output
        )

    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_025_config_load(self, mock_load_cfg):
        """test _config_load bogus ca_handler"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"handler_file": "foo"}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._config_load()
        self.assertIn(
            "CRITICAL:test_a2c:Loading CAhandler configured in cfg failed with err: 'NoneType' object has no attribute 'loader'",
            lcm.output,
        )

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_026_config_load(self, mock_load_cfg, mock_imp):
        """test _config_load missing ca_handler"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"handler_file": "foo"}
        mock_load_cfg.return_value = parser
        mock_imp.return_value = Mock()
        self.trigger._config_load()
        self.assertTrue(self.trigger.cahandler)

    @patch("importlib.import_module")
    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_027_config_load(self, mock_load_cfg, mock_imp):
        """test _config_load missing ca_handler"""
        parser = configparser.ConfigParser()
        parser["CAhandler"] = {"foo": "bar"}
        mock_load_cfg.return_value = parser
        mock_imp.return_value = Mock()
        self.trigger._config_load()
        self.assertTrue(self.trigger.cahandler)

    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_028_config_load(self, mock_load_cfg):
        """test _config_load empty config"""
        parser = configparser.ConfigParser()
        parser["Order"] = {"tnauthlist_support": False}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._config_load()
        self.assertFalse(self.trigger.tnauthlist_support)

    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_029_config_load(self, mock_load_cfg):
        """test _config_load empty config"""
        parser = configparser.ConfigParser()
        parser["Order"] = {"tnauthlist_support": True}
        mock_load_cfg.return_value = parser
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._config_load()
        self.assertTrue(self.trigger.tnauthlist_support)

    @patch("acme2certifier.acme_srv.trigger.ca_handler_load")
    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_030_config_load(self, mock_load_cfg, mock_cahandler_load):
        """test _config_load()"""
        parser = configparser.ConfigParser()
        parser["Foo"] = {"foo": "bar"}
        mock_load_cfg.return_value = parser
        mock_cahandler_load.return_value = "foo"
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.trigger._config_load()
        self.assertIn(
            "CRITICAL:test_a2c:Failed to load CA handler module: 'str' object has no attribute 'CAhandler'",
            lcm.output,
        )

    @patch("acme2certifier.acme_srv.trigger.ca_handler_load")
    @patch("acme2certifier.acme_srv.trigger.load_config")
    def test_031_config_load(self, mock_load_cfg, mock_cahandler_load):
        """test _config_load()"""
        parser = configparser.ConfigParser()
        parser["Foo"] = {"foo": "bar"}
        mock_load_cfg.return_value = parser
        self.trigger._config_load()
        self.assertTrue(self.trigger.cahandler)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""Unit tests for handler-config deprecation helpers."""

import logging
import unittest
import warnings
from unittest.mock import patch

from acme2certifier import compat


class TestCompatDeprecation(unittest.TestCase):
    """Tests for acme2certifier.compat warnings."""

    def setUp(self):
        compat._WARNED.clear()

    def tearDown(self):
        compat._WARNED.clear()

    def test_warn_file_config_deprecated(self):
        logger = logging.getLogger("test_compat")
        with warnings.catch_warnings(record=True) as caught:
            warnings.simplefilter("always")
            with self.assertLogs("test_compat", level="WARNING") as lcm:
                compat.warn_file_config_deprecated(
                    logger,
                    "handler_file",
                    "handler_module",
                    "acme2certifier.cahandlers.openssl_ca_handler",
                )
        self.assertEqual(len(caught), 1)
        self.assertIn("handler_file is deprecated", str(caught[0].message))
        self.assertTrue(any("handler_file is deprecated" in line for line in lcm.output))

    def test_warn_default_ca_handler(self):
        logger = logging.getLogger("test_compat")
        with warnings.catch_warnings(record=True) as caught:
            warnings.simplefilter("always")
            with self.assertLogs("test_compat", level="WARNING") as lcm:
                compat.warn_default_ca_handler(logger)
        self.assertEqual(len(caught), 1)
        self.assertIn("acme_srv.ca_handler", str(caught[0].message))
        self.assertTrue(any("acme_srv.ca_handler" in line for line in lcm.output))


class TestPluginLoaderFileDeprecated(unittest.TestCase):
    """plugin_loader warns on handler_file."""

    def setUp(self):
        compat._WARNED.clear()
        self.logger = logging.getLogger("test_compat_plugin")

    def tearDown(self):
        compat._WARNED.clear()

    @patch("importlib.util")
    def test_ca_handler_file_deprecated(self, mock_util):
        from acme2certifier.acme_srv.helpers.plugin_loader import ca_handler_load

        mock_util.module_from_spec = lambda spec: "foo"
        config_dic = {"CAhandler": {"handler_file": "foo"}}
        with warnings.catch_warnings(record=True) as caught:
            warnings.simplefilter("always")
            with self.assertLogs("test_compat_plugin", level="WARNING") as lcm:
                self.assertEqual("foo", ca_handler_load(self.logger, config_dic))
        self.assertTrue(
            any("handler_file is deprecated" in str(w.message) for w in caught)
        )
        self.assertTrue(any("handler_file is deprecated" in line for line in lcm.output))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""Directory class"""

# pylint: disable=e0401, r0913, r1705

from __future__ import print_function
import uuid
import json
from typing import Dict, Optional, List, Tuple
from dataclasses import dataclass, field
from .version import __version__, __dbversion__
from .helper import (
    load_config,
    ca_handler_load,
    config_profile_load,
    config_async_mode_load,
)
from .db_handler import DBstore

GH_HOME = "https://github.com/grindsa/acme2certifier"


@dataclass
class DirectoryConfig:
    """Configuration dataclass for Directory settings and parameters."""

    supress_version: bool = False
    db_check: bool = False
    suppress_product_information: bool = False
    tos_url: Optional[str] = None
    url_prefix: str = ""
    home: str = GH_HOME
    caaidentities: List[str] = field(default_factory=list)
    profiles: Dict = field(default_factory=dict)
    eab: bool = False
    eab_strict_mode: bool = True
    acme_url: Optional[str] = None
    profiles_sync: bool = False
    profiles_sync_interval: int = 604800  # default: 7 days
    async_mode: bool = False


class DirectoryRepository:
    """Repository for all Directory-related database access."""

    def __init__(self, dbstore: object, logger: object) -> None:
        """Initialize DirectoryRepository with dbstore and logger."""
        self.dbstore = dbstore
        self.logger = logger

    def get_db_version(self) -> Tuple[Optional[str], Optional[str]]:
        """Get the current database version from the DBstore."""
        try:
            return self.dbstore.dbversion_get()
        except Exception as err:
            self.logger.critical(
                "Database error: failed to check database version: %s", err
            )
            return None, None

    def profile_list_get(self) -> List[Dict[str, object]]:
        """Get the list of profiles from the database."""
        try:
            profiles = self.dbstore.hkparameter_get("profiles")
        except Exception as err:
            self.logger.critical("Database error: failed to get profile list: %s", err)
            return []
        if profiles:
            try:
                return json.loads(profiles)
            except Exception as err_:
                self.logger.error(
                    "Error when loading the profiles parameter from database: %s", err_
                )
                return []
        return []

    def profile_list_set(self, data_dic: Dict[str, object]) -> None:
        """Set the list of profiles in the database."""
        try:
            self.dbstore.hkparameter_add(data_dic)
        except Exception as err:
            self.logger.critical("Database error: failed to set profile list: %s", err)


class Directory:
    """Main handler for ACME Directory logic, configuration, and response building."""

    def __init__(
        self,
        debug: Optional[object] = None,
        srv_name: Optional[str] = None,
        logger: Optional[object] = None,
    ) -> None:
        """Initialize Directory with configuration, repository, and logger."""
        self.server_name = srv_name
        self.logger = logger
        self.dbstore = DBstore(debug, self.logger)
        self.repository = DirectoryRepository(self.dbstore, self.logger)
        self.config = DirectoryConfig()
        self.cahandler = None
        self.version = __version__
        self.dbversion = __dbversion__

    def __enter__(self) -> "Directory":
        """Enter context manager for Directory."""
        self._load_configuration()
        return self

    def __exit__(self, *args) -> None:
        """Exit context manager for Directory."""
        # pylint: disable=w0107
        pass

    def _load_configuration(self) -> None:
        """Load and parse all Directory configuration from file and environment."""
        self.logger.debug("Directory._load_configuration()")
        config_dic = load_config(self.logger, "Directory")

        self._parse_directory_section(config_dic)
        self._parse_booleans(config_dic)
        self._parse_eab_and_profiles(config_dic)
        self._parse_cahandler_section(config_dic)
        self._load_ca_handler(config_dic)
        self.config.async_mode = config_async_mode_load(
            self.logger, config_dic, self.dbstore.type
        )
        self.logger.debug("Directory._load_configuration() ended")

    def _parse_directory_section(self, config_dic: object) -> None:
        """Parse the [Directory] section for basic config values."""
        if "Directory" in config_dic:
            cfg_dic = dict(config_dic["Directory"])
            self.config.tos_url = cfg_dic.get("tos_url", None)
            self.config.url_prefix = cfg_dic.get("url_prefix", "")
            self.config.home = cfg_dic.get("home", GH_HOME)
            tmp_caaidentities = config_dic.get(
                "Directory", "caaidentities", fallback=None
            )
            if tmp_caaidentities:
                self.config.caaidentities = self._parse_caaidentities(tmp_caaidentities)

    def _parse_caaidentities(self, value: str) -> List[str]:
        """Parse the caaIdentities config value as JSON or fallback to list."""
        try:
            return json.loads(value)
        except Exception as err_:
            if "[" not in value and '"' not in value:
                return [value]
            else:
                self.logger.error(
                    "Error when loading the caaIdentities parameter from config: %s",
                    err_,
                )
                return []

    def _parse_booleans(self, config_dic: object) -> None:
        """Parse boolean config values for Directory settings."""
        for key, attr in [
            ("supress_version", "supress_version"),
            ("db_check", "db_check"),
            ("suppress_product_information", "suppress_product_information"),
        ]:
            try:
                setattr(
                    self.config,
                    attr,
                    config_dic.getboolean(
                        "Directory", key, fallback=getattr(self.config, attr)
                    ),
                )
            except Exception as err_:
                self.logger.error("%s not set: %s", key, err_)

    def _parse_eab_and_profiles(self, config_dic: object) -> None:
        """Parse EAB handler and profile configuration."""
        if "EABhandler" in config_dic and (
            "eab_handler_file" in config_dic["EABhandler"]
            or "eab_handler_module" in config_dic["EABhandler"]
        ):
            self.config.eab = True
            self.config.eab_strict_mode = self._parse_eab_strict_mode(config_dic)
        self.config.profiles = config_profile_load(self.logger, config_dic)

    def _parse_eab_strict_mode(self, config_dic: object) -> bool:
        """Parse strict EAB account registration mode from [EABhandler]."""
        try:
            return config_dic.getboolean(
                "EABhandler",
                "eab_strict_mode",
                fallback=self.config.eab_strict_mode,
            )
        except Exception:
            try:
                eab_cfg = config_dic["EABhandler"] if "EABhandler" in config_dic else {}
                raw_value = eab_cfg.get("eab_strict_mode", None)
            except Exception:
                raw_value = None
            if raw_value is None:
                return self.config.eab_strict_mode
            if isinstance(raw_value, bool):
                return raw_value

            raw = str(raw_value).strip().lower()
            if raw in ["1", "true", "yes", "on"]:
                return True
            if raw in ["0", "false", "no", "off"]:
                return False

            self.logger.warning(
                "Invalid eab_strict_mode value '%s'; fallback to default '%s'",
                raw_value,
                self.config.eab_strict_mode,
            )
            return self.config.eab_strict_mode

    def _parse_cahandler_section(self, config_dic: object) -> None:
        """Parse the [CAHandler] section for ACME URL and profile sync settings."""
        self.logger.debug("Directory._parse_cahandler_section()")
        if "CAhandler" in config_dic:
            cfg_dic = dict(config_dic["CAhandler"])
            self.config.acme_url = cfg_dic.get("acme_url", None)
            try:
                self.config.profiles_sync = config_dic.getboolean(
                    "CAhandler",
                    "profiles_sync",
                    fallback=self.config.profiles_sync,
                )
            except Exception as err_:
                self.logger.error("profiles_sync not set: %s", err_)

            self._validate_profiles_sync()
            self._set_profiles_sync_interval(config_dic)

        self.logger.debug("Directory._parse_cahandler_section() ended")

    def _validate_profiles_sync(self) -> None:
        if not self.config.profiles_sync:
            return
        if self.config.profiles:
            self.logger.error(
                "Profiles are configured via acme_srv.cfg. Disabling profile sync."
            )
            self.config.profiles_sync = False
        elif not self.config.acme_url:
            self.logger.error("profiles_sync is set but no acme_url configured.")
            self.config.profiles_sync = False

    def _set_profiles_sync_interval(self, config_dic: object) -> None:
        if not self.config.profiles_sync:
            return
        try:
            self.config.profiles_sync_interval = config_dic.getint(
                "CAhandler",
                "profiles_sync_interval",
                fallback=self.config.profiles_sync_interval,
            )
        except Exception as err_:
            self.logger.error("profiles_sync_interval not set: %s", err_)
        self.logger.debug(
            "Directory._parse_cahandler_section(): profiles_sync is enabled. Interval: %s seconds",
            self.config.profiles_sync_interval,
        )

    def _load_ca_handler(self, config_dic: object) -> None:
        """Load the CA handler module as configured."""
        ca_handler_module = ca_handler_load(self.logger, config_dic)
        if ca_handler_module:
            self.cahandler = ca_handler_module.CAhandler
        else:
            self.logger.critical("No ca_handler loaded")

    def _build_meta_information(self) -> Dict[str, object]:
        """Build the meta information dictionary for the directory response."""
        self.logger.debug("Directory._build_meta_information()")
        meta_dic = {}
        if not self.config.suppress_product_information:
            meta_dic = {
                "home": self.config.home,
                "author": "grindsa <grindelsack@gmail.com>",
                "name": "acme2certifier",
            }
            if not self.config.supress_version:
                meta_dic["version"] = self.version
        else:
            if self.config.home != GH_HOME:
                meta_dic["home"] = self.config.home
        if self.config.tos_url:
            meta_dic["termsOfService"] = self.config.tos_url
        if self.config.caaidentities:
            meta_dic["caaIdentities"] = self.config.caaidentities
        if self.config.profiles:
            meta_dic["profiles"] = self.config.profiles
        if self.config.eab:
            meta_dic["externalAccountRequired"] = self.config.eab_strict_mode
        self.logger.debug("Directory._build_meta_information() ended")
        return meta_dic

    def _build_directory_response(self) -> Dict[str, object]:
        """Build the full directory response dictionary for the ACME directory endpoint."""
        self.logger.debug("Directory._build_directory_response()")
        d_dic = {
            "newAuthz": self.server_name + self.config.url_prefix + "/acme/new-authz",
            "newNonce": self.server_name + self.config.url_prefix + "/acme/newnonce",
            "newAccount": self.server_name
            + self.config.url_prefix
            + "/acme/newaccount",
            "newOrder": self.server_name + self.config.url_prefix + "/acme/neworders",
            "revokeCert": self.server_name
            + self.config.url_prefix
            + "/acme/revokecert",
            "keyChange": self.server_name + self.config.url_prefix + "/acme/key-change",
            "renewalInfo": self.server_name
            + self.config.url_prefix
            + "/acme/renewal-info",
            "meta": self._build_meta_information(),
        }
        if self.config.db_check:
            version, _script_name = self.repository.get_db_version()
            if version == self.dbversion:
                d_dic["meta"]["db_check"] = "OK"
            else:
                self.logger.error(
                    "Database schema mismatch detected: detected: %s/ expected: %s",
                    version,
                    self.dbversion,
                )
                d_dic["meta"]["db_check"] = "NOK"
        # generate random key in json as recommended by LE
        d_dic[uuid.uuid4().hex] = (
            "https://community.letsencrypt.org/t/adding-random-entries-to-the-directory/33417"
        )
        self.logger.debug("Directory._build_directory_response() ended")
        return d_dic

    def get_directory_response(self) -> Dict[str, object]:
        """Public method to get the ACME directory response, including CA handler checks."""
        self.logger.debug("Directory.get_directory_response()")
        error = None

        if self.cahandler:

            with self.cahandler(None, self.logger) as ca_handler:
                if hasattr(ca_handler, "handler_check"):
                    error = ca_handler.handler_check()
                if (
                    self.config.profiles_sync
                    and hasattr(ca_handler, "synchronize_profiles")
                    and not error
                ):
                    self.config.profiles = ca_handler.synchronize_profiles(
                        self.repository,
                        self.config.acme_url,
                        self.config.profiles_sync_interval,
                        self.config.async_mode,
                    )

        else:
            error = "No handler loaded"

        if not error:
            d_dic = self._build_directory_response()
        else:
            self.logger.critical(
                "CA handler error during get_directory_response: %s", error
            )
            d_dic = {"error": "error in ca_handler configuration"}
        return d_dic

    def directory_get(self) -> Dict[str, object]:
        """return response to ACME directory call"""
        self.logger.debug("Directory.directory_get()")
        return self.get_directory_response()

    def servername_get(self) -> str:
        """dumb function to return servername"""
        self.logger.debug("Directory.servername_get()")
        return self.server_name
